part of tecfy_core_client_package;

class DatabaseService {
  static Database _db;
  static String pathString;
  static String _dbPath = 'main.db';
  static LocalStorage storage;
  static bool isInit = false;
  static init() async {
    try {
      if (kIsWeb ?? false) {
        storage = new LocalStorage('localDB.json');
      } else if ((Platform.isAndroid || Platform.isIOS)) {
        pathString = (await path.getApplicationDocumentsDirectory()).path + '/';
        //await deleteDatabase(_dbPath);
        _db = await openDatabase(_dbPath,
            version: 2,
            onCreate: (db, v) => _createDatabase(db, 0, v),
            onUpgrade: _createDatabase,
            onDowngrade: _createDatabase);
        print('db has been inited successfully !!!!!!!!!!!!!');
      } else {
        pathString = "c:\\data\\";
      }
    } catch (err) {
      // cannot get path means it's windows
      pathString = "c:\\data\\";
      print('xxxxxxxxxxxxxx  error inital database xxxxxxxxxxxx');
      print(err);
    }
    isInit = true;
  }

  static Future<bool> exeBatch(Future<bool> Function(Batch) process,
      [bool continueOnError = false]) async {
    try {
      if (_db == null) return false;
      var batch = _db.batch();
      if (await process(batch)) {
        batch.commit(continueOnError: continueOnError);
      }
      return true;
    } catch (err) {
      return false;
    }
  }

  static Future<List<Map<String, dynamic>>> rawQuery(String sql,
      [List<dynamic> arguments]) {
    if (_db == null || !_db.isOpen) {
      return Future.value([]);
    }
    return _db.rawQuery(sql, arguments);
  }

  static Future<int> insert(String table, Map<String, dynamic> values,
      {String nullColumnHack, ConflictAlgorithm conflictAlgorithm}) async {
    if (_db == null || !_db.isOpen) {
      print(
          'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX   db is not oppened!');
      return 0;
    }
    await _db.insert(table, values,
        nullColumnHack: nullColumnHack, conflictAlgorithm: conflictAlgorithm);
    return 10;
  }

  static Future<int> delete(String table,
      {String where, List<dynamic> whereArgs}) {
    if (_db == null || !_db.isOpen) {
      return Future.value(0);
    }
    return _db.delete(table, where: where, whereArgs: whereArgs);
  }

  static Future<int> update(String table, Map<String, dynamic> values,
      {String where,
      List<dynamic> whereArgs,
      ConflictAlgorithm conflictAlgorithm}) {
    if (_db == null || !_db.isOpen) {
      return Future.value(0);
    }
    return _db.update(table, values,
        where: where,
        whereArgs: whereArgs,
        conflictAlgorithm: conflictAlgorithm);
  }

  static Future<List<Map<String, dynamic>>> query(String table,
      {bool distinct,
      List<String> columns,
      String where,
      List<dynamic> whereArgs,
      String groupBy,
      String having,
      String orderBy,
      int limit,
      int offset}) {
    if (_db == null || !_db.isOpen) {
      return Future.value([]);
    }
    return _db.query(table,
        distinct: distinct,
        columns: columns,
        where: where,
        whereArgs: whereArgs,
        groupBy: groupBy,
        limit: limit,
        orderBy: orderBy,
        offset: offset);
  }

  static Future<void> _createDatabase(
      Database _db, int _fromVerison, int _toVersion) async {
    try {
      // var fmVersion =0;
      // var appVersion =0;

      if (_fromVerison <= 1) {
        await _db.execute(
            'CREATE TABLE Message (id TEXT PRIMARY KEY, fromUserId TEXT,toUserId TEXT,body TEXT, fromName TEXT,toName TEXT, time INTEGER,imSender INTEGER)');
        await _db.execute(
            'CREATE TABLE Notification (id TEXT PRIMARY KEY, titleAr TEXT,titleEn TEXT,messageAr TEXT, messageEn TEXT, time INTEGER, seen INTEGER, type INTEGER, payload TEXT)');
        await _db.execute(
            'CREATE TABLE Entity (id TEXT PRIMARY KEY, entity TEXT,body TEXT, timeStamp INTEGER, recordNumber INTEGER,isPosted INTEGER)');
        await _db.execute(
            'CREATE TABLE DeleteLog (id TEXT PRIMARY KEY, entity TEXT, timeStamp INTEGER,isPosted INTEGER)');

        await _db.execute(
            'CREATE INDEX INX_Entity_entity ON Entity(entity,timeStamp)');
        await _db
            .execute('CREATE INDEX INX_Entity_isPosted ON Entity(isPosted)');
        await _db.execute(
            'CREATE INDEX INX_DeleteLog_isPosted ON DeleteLog(isPosted)');
      }
      await TecfyCoreApp.updateSqlLiteDb(_db, _fromVerison, _toVersion);
    } catch (err) {
      print(err);
    }
  }

  static Future<void> deleteDatabaseFile() async {
    try {
      await deleteDatabase(_dbPath);
    } catch (err) {
      print('xxxxxxxxxxxxxx  error deleting database xxxxxxxxxxxx');
      print(err);
    }
  }

  static Future<bool> saveEntity(String entity, List<dynamic> data,
      [bool isPosted = true]) async {
    if (kIsWeb) return false;
    print('Start ----------------------- entity saved $entity ${data.length}');
    List<Map<String, dynamic>> dataList = [];
    for (var d in data) {
      var newData = Map<String, dynamic>();
      if (d['id'] == null) {
        d['id'] = Uuid().v4().replaceAll('-', '');
      }
      newData['id'] = d['id'];
      newData['entity'] = entity;
      if (d['timeStamp'] != null) {
        if (d['timeStamp'] is DateTime)
          newData['timeStamp'] =
              (d['timeStamp'] as DateTime).millisecondsSinceEpoch;
        else
          newData['timeStamp'] = BaseModel.parseUTC(d['timeStamp'].toString())
              .millisecondsSinceEpoch;
      } else {
        newData['timeStamp'] = DateTime.now().millisecondsSinceEpoch;
      }
      newData['recordNumber'] = d['recordNumber'];
      newData['isPosted'] = isPosted ? 1 : 0;
      newData['body'] = json.encode(d, toEncodable: EntityService.customEncode);
      dataList.add(newData);
    }

    var ids = dataList.map<String>((d) => d['id']).join("','");

    await DatabaseService.delete('Entity',
        where: "isPosted = ? and id in ('$ids')", whereArgs: [1]);
    var result = await DatabaseService.exeBatch((batch) async {
      for (var item in dataList) {
        batch.insert('Entity', item);
      }
      return true;
    });
    print('---------------- entity saved $entity ${dataList.length} $result');
    return result;
  }

  static Future<List<dynamic>> getEntities(String entity,
      {String where = '',
      List<dynamic> whereArgs,
      String orderBy = 'timeStamp',
      int offset,
      int limit}) async {
    if (where.isEmpty)
      where = "entity = '$entity'";
    else
      where = "entity = '$entity' and $where";

    var items = (await DatabaseService.query('Entity',
            where: where,
            whereArgs: (whereArgs?.length ?? 0) == 0 ? null : whereArgs,
            orderBy: orderBy,
            offset: offset,
            limit: limit))
        .map((d) => json.decode(d['body']))
        .toList();
    print(
        '----------------------------- get $entity from local DB (${items.length})');

    return items;
  }

  static Future<dynamic> getEntityById(String id) async {
    if (kIsWeb) return null;
    var items = (await DatabaseService.query('Entity',
            where: 'id = ?', whereArgs: [id]))
        .map((d) => jsonDecode(d['body']))
        .toList();

    return items.length > 0 ? items[0] : null;
  }

  static Future<OperationResult> deleteEntityById(
      String entity, String id) async {
    if (kIsWeb) return OperationResult(success: true);
    await DatabaseService.exeBatch((b) async {
      b.insert('DeleteLog', {
        'id': id,
        'timeStamp': DateTime.now().millisecondsSinceEpoch,
        'entity': entity,
        'isPosted': 0
      });
      b.delete('Entity', where: 'id = ?', whereArgs: [id]);
      await b.commit();
      return true;
    });

    // var items = (await DatabaseService.delete('Entity',
    //     where: 'id = ?', whereArgs: [id]));

    return OperationResult(success: true);
  }

  static Future<OperationResult> deleteEntityBulk(List<String> ids) async {
    if (kIsWeb) return OperationResult(success: true);
    await DatabaseService.exeBatch((b) async {
      for (var id in ids) {
        b.delete('Entity', where: 'id = ?', whereArgs: [id]);
      }
      await b.commit();
      return true;
    });
    return OperationResult(success: true);
  }

  // static Future<OperationResult> insert(
  //     String table, Map<String, dynamic> data) async {
  //   if (db == null || !db.isOpen) return OperationResult(success: false);
  //   var res = await db.insert(table, data);
  //   return OperationResult(success: res > 0);
  // }

  static Future<OperationResult> writeCollection(
      String entity, List<dynamic> data) async {
    try {
      if (kIsWeb) {
        storage.setItem(entity, data);
      } else {
        var file = File("$pathString$entity.json");
        await file.writeAsString(json.encode(data));
      }
      return OperationResult(success: true);
    } catch (err) {
      print('XXXXXXXXXXXXXXXXX  write local collection ($entity) $err');
      return OperationResult(success: false, message: err.toString());
    }
  }

  static Future<OperationResult<List<dynamic>>> readCollection(
      String entity) async {
    try {
      while (!isInit) await Future.delayed(Duration(milliseconds: 100));

      // print('--------------- read local collection ($entity)------------------');
      List<dynamic> data;
      if (kIsWeb) {
        data = storage.getItem(entity);
      } else {
        var file = File("$pathString$entity.json");
        data = json.decode(await file.readAsString());
      }
      if (data == null) return OperationResult(success: false);
      return OperationResult(success: true, data: data);
    } catch (err) {
      //print('XXXXXXXXXXXXXXXXX  read local collection ($entity) ("$pathString$entity.json") $err');
      return OperationResult(success: false, message: err.toString());
    }
  }
}
