part of tecfy_core_client_package;

class EntityService {
  static List<EntityCategory> categories;
  //static List<Entity> entities;
  static Map<String, Entity> entities;
  static Map<String, List<dynamic>> dataCache = {};
  static int pageSize = 15;

  static Future initEntities() async {
    try {
      var result = await ApiService.httpGetDynamic('entity/GetEntities');
      if (!result.success) {
        result.data = jsonDecode(ConfigService.getValueString('Entity'));
        result.success = (result.data != null);
      }
      if (result.success) {
        print('-----------  GetEntities ------------ ${result.data.length}');
        ConfigService.setValueString('Entity', jsonEncode(result.data));
        //print(result.data);
        List<Entity> entList = [];
        EntityService.entities = {};
        for (var cat in result.data) {
          var ent = Entity.fromMap(cat);
          entList.add(ent);
          EntityService.entities[ent.dbName] = ent;
        }

        print(EntityService.entities.keys.length);
        for (var c in EntityService.categories) {
          c.entities = entList
              .where((e) => c.entities.any((ee) => ee.dbName == e.dbName))
              .toList();
        }
        print('categories is ${EntityService.categories.length}');
      }
    } catch (err) {
      print('XXXXXXXXXXXXXXX GetEntities $err');
      throw "init entities Error: $err";
    }
  }

  Future<OperationResult<List<dynamic>>> getData(Entity entity,
      {bool forceFromServer = false,
      List<EntityFilter> filter,
      int pageIndex = 0,
      bool activeOnly = false,
      bool allRecords = false,
      String parentId,
      bool getFromServerIfNoLocal = true,
      bool filterParentId = false}) async {
    try {
      if (entity == null) {
        return OperationResult(
            success: false, message: 'GetData: Entity cannot be null');
      }
      print('start get data');
      if (!kIsWeb &&
          (ConnectionService.isOffline ||
              entity.isLocal ||
              (!forceFromServer &&
                  (filter == null || filter.length == 0) &&
                  !activeOnly &&
                  (filter == null || filter.length == 0)))) {
        // var dbResult = await DatabaseService.readCollection(entity.dbName);
        var dbResult = await DatabaseService.getEntities(entity.dbName);
        if (dbResult != null && (!getFromServerIfNoLocal || dbResult.length > 0)) {
          dbResult.sort((a, b) => (a['sortId'] ?? 1) - (b['sortId'] ?? 1));
          return OperationResult<List<dynamic>>(success: true, data: dbResult);
        }
        if (entity.isLocal)
          return OperationResult<List<dynamic>>(
              success: false, message: 'No Records in Local Database!');
      }
      var pagingQuery =
          '?skip=${pageIndex * pageSize}&count=${allRecords ? 10000 : pageSize}&activeOnly=$activeOnly&filterParentId=$filterParentId';
      if (parentId != null) pagingQuery += '&parentId=$parentId';

      var result = await ApiService.httpGetDynamic('entity/get/${entity.dbName}' +
          ((filter?.length ?? 0) > 0
              ? '$pagingQuery&filter=${jsonEncode(filter.map<Map>((f) => f.toJson()).toList())}'
              : pagingQuery));
      if (!result.success) {
        print(result.success);
        print(result.message);
        return OperationResult<List<dynamic>>(
            success: false, message: result.message);
      }
      // if(cache) dataCache[entity] =result.data;
      // if (pageIndex == 0)
      //   await DatabaseService.writeCollection(entity.dbName, result.data);
      await DatabaseService.saveEntity(entity.dbName, result.data);
      return OperationResult<List<dynamic>>(success: true, data: result.data);
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXX EntotyService.getData');
      print(err);
      return OperationResult<List<dynamic>>(
          success: false, message: err.toString());
    }
  }

  Future<dynamic> getOne(String entity, dynamic id,
      {bool forView = false, bool fromLocal = false}) async {
    OperationResult result;
    if (fromLocal && !kIsWeb) {
      result = OperationResult(success: false);
    } else {
      result = await ApiService.httpGetDynamic(
          'entity/GetById/$entity/$id?view=$forView');
    }
    if (!result.success && !kIsWeb) {
      var dbResult = await DatabaseService.getEntityById(id);
      if (dbResult != null) return dbResult;
      print(result.success);
      print(result.message);
      return null;
    }
    await DatabaseService.saveEntity(entity, [result.data]);
    return result.data;
  }

  Future<OperationResult> delete(Entity entity, dynamic id) async {
    // if (entity.isLocal) {
    //   var localItems = await DatabaseService.readCollection(entity.dbName);
    //   if (!localItems.success) return localItems;
    //   localItems.data.removeWhere((i) => i['id'] == id);
    //   var saveResult =
    //       await DatabaseService.writeCollection(entity.dbName, localItems.data);
    //   if (saveResult.success && entity.dbName == '_ServerConnections') {
    //     ConnectionService.reloadConnections();
    //   }
    //   return saveResult;
    // }

    if (ConnectionService.isOffline || entity.isLocal) {
      return await DatabaseService.deleteEntityById(entity.dbName, id);
    }
    if (!kIsWeb) {
      var oldValue = await getOne(entity.dbName, id, fromLocal: true);
      if (oldValue == null || oldValue['isPosted'] == false) {
        return await DatabaseService.deleteEntityById(entity.dbName, id);
      }
    }

    var result =
        await ApiService.httpDeleteDynamic('entity/${entity.dbName}/$id');
    if (!result.success) {
      print(result.success);
      print(result.message);
      return result;
    } else {
      return await DatabaseService.deleteEntityById(entity.dbName, id);
    }
  }

  static Entity getEntity(String name) {
    if (name == '_TestEntity') {
      return Entity.fromMap({
        'dbName': '_TestEntity',
        'nameAr': 'TestEntity',
        'nameEn': 'TestEntity',
        'isLocal': true,
        'permission': {
          'canView': true,
          'canUpdate': true,
          'canAdd': true,
          'canDelete': true
        },
        'properties': [
          {
            'dbName': 'name',
            'nameAr': 'Name',
            'nameEn': 'Name',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'description',
            'nameAr': 'description',
            'nameEn': 'description',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'area',
            'nameAr': 'Area',
            'nameEn': 'Area',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.MarkImage.index,
            'fullWidth': true,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          }
        ]
      });
    }
    if (name == '_ServerConnections') {
      return Entity.fromMap({
        'isLocal': true,
        'nameAr': 'قنوات الاتصال',
        'nameEn': 'Server Connections',
        'dbName': '_ServerConnections',
        'permission': {
          'canView': true,
          'canUpdate': true,
          'canAdd': true,
          'canDelete': true
        },
        'properties': [
          {
            'dbName': 'name',
            'nameAr': 'الاسم',
            'nameEn': 'Name',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': true,
            'isRequired': true,
            'isSearchable': true,
            'isDisplayField': true
          },
          {
            'dbName': 'server',
            'nameAr': 'عنوان الخادم',
            'nameEn': 'Server',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'sortId',
            'nameAr': 'الترتيب',
            'nameEn': 'Sort Id',
            'valueType': ValueTypes.Number.index,
            'propertyType': PropertyTypes.Number.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'version',
            'nameAr': 'الاصدار',
            'nameEn': 'Version',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Text.index,
            'fullWidth': false,
            'isEdit': false,
            'isGrid': true,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'lastActive',
            'nameAr': 'اخر وقت للفحص',
            'nameEn': 'Last Active',
            'valueType': ValueTypes.Number.index,
            'propertyType': PropertyTypes.DateTime.index,
            'fullWidth': false,
            'isEdit': false,
            'isGrid': true,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'isCloud',
            'nameAr': 'هل سحابي؟',
            'nameEn': 'Is Cloud',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'isHTTPS',
            'nameAr': 'HTTPS',
            'nameEn': 'HTTPS',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'active',
            'nameAr': 'فعال',
            'nameEn': 'Active',
            'valueType': ValueTypes.Boolean.index,
            'propertyType': PropertyTypes.Boolean.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': true,
            'isSearchable': false,
            'isDisplayField': false
          },
          {
            'dbName': 'colorString',
            'nameAr': 'اللون',
            'nameEn': 'Color',
            'valueType': ValueTypes.Text.index,
            'propertyType': PropertyTypes.Color.index,
            'fullWidth': false,
            'isEdit': true,
            'isGrid': false,
            'isRequired': false,
            'isSearchable': false,
            'isDisplayField': false
          },
        ],
      });
    }
    return entities[lowerFirst(name)];
  }

  static OperationResult validateEntity(Entity entity, dynamic data) {
    print('Validate Entity : ${entity.dbName}');
    try {
      for (var prop in entity.properties
          .where((e) => e.isEdit == true && e.readOnly == false)) {
        if (prop.propertyType == PropertyTypes.OtherEntityMany) {
          var subList = data[prop.dbName];
          if (prop.isRequired && (subList == null || subList.length == 0))
            return OperationResult(
                success: false,
                message: 'Add Atleast one record: ${prop.nameEn}');
          else if (subList != null)
            for (var sub in subList) {
              var result = validateEntity(
                  entities[(prop as PropertyOtherEntityMany).entityName], sub);
              if (!result.success) return result;
            }
        } else {
          var result = prop.isValid(data[prop.dbName]);
          if (!result.success) {
            print('------Error validating: ${prop.dbName}');
            print(data[prop.dbName]);
            print(result.message);
            return result;
          }
        }
      }
      return OperationResult(success: true);
    } catch (err, stack) {
      return OperationResult(
          success: false,
          message: 'Error in Validation ${entity.dbName}: ${err.toString()}');
    }
  }

  static Future<OperationResult> _addEntity(Entity entity, data) async {
    print('+++++++++++++++++++++ Start Add $entity');
    // if (entity.isLocal) {
    //   var localItems = await DatabaseService.readCollection(entity.dbName);
    //   if (!localItems.success) localItems.data = [];
    //   if (data['id'] == null) data['id'] = Uuid().v4().replaceAll('-', '');
    //   localItems.data.add(data);
    //   var saveResult =
    //       await DatabaseService.writeCollection(entity.dbName, localItems.data);
    //   if (saveResult.success) ConnectionService.reloadConnections();
    //   return saveResult;
    // }

    if (ConnectionService.isOffline || entity.isLocal) {
      var success =
          await DatabaseService.saveEntity(entity.dbName, [data], false);
      return OperationResult(success: success);
    }

    var result = await ApiService.httpPostEx('entity/${entity.dbName}',
        json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
    if (result.success) {
      await DatabaseService.saveEntity(entity.dbName, [result.data]);
      return result;
    } else {
      return result;
    }
  }

  static Future<OperationResult> _updateEntity(Entity entity, data) async {
    print('+++++++++++++++++++++ Start Update ${entity.dbName}');
    // if (entity.isLocal) {
    //   var localItems = await DatabaseService.readCollection(entity.dbName);
    //   if (!localItems.success) return localItems;
    //   if (data['id'] == null) data['id'] = Uuid().v4().replaceAll('-', '');
    //   localItems.data.removeWhere((element) => element['id'] == data['id']);
    //   localItems.data.add(data);
    //   localItems.data.sort((a, b) => a['sortId'] ?? 0 - b['sortId'] ?? 0);
    //   var saveResult =
    //       await DatabaseService.writeCollection(entity.dbName, localItems.data);
    //   if (saveResult.success) ConnectionService.reloadConnections();
    //   return saveResult;
    // }

    if (ConnectionService.isOffline || entity.isLocal) {
      var success =
          await DatabaseService.saveEntity(entity.dbName, [data], false);
      return OperationResult(success: success);
    }

    var result = await ApiService.httpPutEx('entity/${entity.dbName}',
        json.encode(data, toEncodable: customEncode).replaceAll('\"', '\''));
    if (result.success) {
      await DatabaseService.saveEntity(entity.dbName, [data]);
      return result;
    } else {
      return result;
    }
  }

  static Future<OperationResult> saveEntity(
      Entity entity, dynamic data, BuildContext context, bool isAddNew) async {
    try {
      if (data['isLocked'] == true) {
        return OperationResult(success: false, message: 'Record is Locked');
      }
      print('+++++++++++++++++++++ Start Save $entity');
      var validationResult = EntityService.validateEntity(entity, data);
      if (!validationResult.success) return validationResult;
      IEntityTransaction service =
          TecfyCoreApp.entityServices[entity.dbName.toLowerCase()];
      OperationResult result;

      if (isAddNew) {
        if (service != null) {
          var serviceResult = await service.beforeAdd(context, data);
          if (!serviceResult.success) {
            return serviceResult;
          }
          if (!serviceResult.skip && serviceResult.success) {
            result = await EntityService._addEntity(entity, serviceResult.data);
          }
        }
        if (result == null) {
          result = await EntityService._addEntity(entity, data);
        }
      } else {
        if (service != null) {
          var serviceResult = await service.beforeUpdate(context, data);
          if (!serviceResult.success) {
            return serviceResult;
          }
          if (!serviceResult.skip && serviceResult.success) {
            result =
                await EntityService._updateEntity(entity, serviceResult.data);
          }
        }
        if (result == null) {
          result = await EntityService._updateEntity(entity, data);
        }
      }

      if (!result.success) {
        return result;
      } else {
        // after add or update
        if (service != null) {
          OperationResultEntityTransaction serviceResult;
          if (isAddNew)
            serviceResult = await service.afterAdd(context, data);
          else
            serviceResult = await service.afterUpdate(context, data);

          return serviceResult;
        }
      }
      return result;
    } catch (err) {
      print('XXXXXXXXXXXXXXX Error Save Entity: $err');
      print(data);
      return OperationResult(success: false, message: 'Error: $err');
    }
  }

  static dynamic customEncode(dynamic item) {
    if (item is DateTime) {
      return intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(item);
    }
    return item;
  }

  static lowerFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  static upperFirst(String s) {
    if (s == null || s.isEmpty) return '';
    return '${s[0].toUpperCase()}${s.substring(1)}';
  }
}
