part of tecfy_core_client_package;

class TecfySyncService {
  static get isOfflineMode {
    return !kIsWeb &&
        (ConnectionService.isOffline ||
            ConnectionService.activeConnection?.isCloud == true);
  }

  static Future<int> getUnpostedCount() async {
    var unPosted = await DatabaseService.rawQuery(
        'select count(*) from Entity where isPosted = 0');
    var deleted = await DatabaseService.rawQuery(
        'select count(*) from DeleteLog where isPosted = 0');
    if (unPosted.length == 0 && deleted.length == 0) return -1;

    return (unPosted.first.values.first + deleted.first.values.first);
  }

  static Future<bool> pushNow() async {
    try {
      var isLoggin = await AccountService.isLogin(null);
      if (kIsWeb || ConnectionService.isOffline || isLoggin.data != true) {
        return true;
      }
      print('++++++++++++++++++++++++++++++++ Start Sync');
      await EntityService().getData(EntityService.getEntity('UOM'),
          forceFromServer: true, allRecords: true, activeOnly: true);
      //====================================================
      //======  delete local
      //====================================================
      print('============= delete local');
      var lastSyncDeletedInt = ConfigService.getValueInt('lastSyncDeleted');
      DateTime lastSyncDeleted = DateTime.now().toUtc();
      if (lastSyncDeletedInt != null && lastSyncDeletedInt > 0)
        lastSyncDeleted =
            DateTime.fromMillisecondsSinceEpoch(lastSyncDeletedInt);
      lastSyncDeleted = lastSyncDeleted.add(Duration(hours: -5));
      var serverDeleted = await ApiService.httpGetDynamic(
          'sync/getDeleted/${intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(lastSyncDeleted)}');
      var idsToDelete =
          serverDeleted.data.map<String>((e) => e['id'].toString()).toList();
      print(idsToDelete);
      await DatabaseService.deleteEntityBulk(idsToDelete);

      (serverDeleted.data as List<dynamic>)
          .sort((a, b) => b['timeStamp'].compareTo(a['timeStamp']));

      ConfigService.setValueInt(
          'lastSyncDeleted',
          BaseModel.parseUTC(serverDeleted.data.first['timeStamp'])
              .millisecondsSinceEpoch);
      //====================================================
      //======  delete from server
      //====================================================
      print('============= delete from server');
      var localDeleted =
          (await DatabaseService.query('DeleteLog', where: 'isPosted = 0'))
              .toList();

      if (localDeleted.length > 0) {
        var sendServerDeleteResult = await ApiService.httpPost(
            'sync/bulkDelete',
            localDeleted.map((e) {
              return {'id': e['id'], 'entity': e['entity']};
            }).toList(),
            true);

        print('+++++++++++++++++++++++++++++++++++ sendServerDeleteResult');
        print(sendServerDeleteResult.data);
        if (sendServerDeleteResult.success &&
            sendServerDeleteResult.data != null &&
            sendServerDeleteResult.data.length > 0) {
          await DatabaseService.exeBatch((b) async {
            for (var i in sendServerDeleteResult.data) {
              b.update('DeleteLog', {'isPosted': 1},
                  where: 'id = ?', whereArgs: [i]);
            }
            b.commit();
            return true;
          });
        }
      }
      //====================================================
      //======  Post to server
      //====================================================
      var unPosted =
          (await DatabaseService.query('Entity', where: 'isPosted = 0'))
              .toList();
      print('SSSSSSSSSSSSS-------------- unPosted to sync ${unPosted.length}');
      if (unPosted.length == 0) return true;

      var modelsResult = await ApiService.httpGetDynamic('Sync/getModels');
      if (!modelsResult.success) return false;

      var models = modelsResult.data;
      print(models);
      //===========
      //== Post un-posted items
      //===========

      for (var model in models) {
        model = EntityService.lowerFirst(model);
        print('=============== start $model');
        for (var item in unPosted.where((e) => e['entity'] == model)) {
          print(item['id']);
          var result = await ApiService.httpPostEx(
              'entity/addupdate/$model',
              jsonEncode(jsonDecode(item['body']),
                      toEncodable: EntityService.customEncode)
                  .replaceAll('\"', '\''));
          if (result.success) {
            await DatabaseService.update('Entity', {'isPosted': 1},
                where: 'id = ?', whereArgs: [item['id']]);
          } else {
            print(result.toString());
          }
        }
      }
      return true;
    } catch (err) {
      print(err);
      return false;
    }
  }

  static Future<bool> syncNow() async {
    if (kIsWeb) {
      return true;
    }
    await pushNow();

    //===========
    //== Load Item category
    //===========
    // var categoryResult =
    //     await ApiService.httpGetDynamic('pos/GetItemCategorySecondry');

    // if (categoryResult.success) {
    //   var items = (categoryResult.data as List<dynamic>)
    //       .map<ItemCategory>((e) => ItemCategory.fromJson(e, null))
    //       .toList();

    //   await DatabaseService.delete('ItemCategory');

    //   await DatabaseService.exeBatch((batch) async {
    //     for (var item in items) {
    //       batch.insert('ItemCategory', item.toJson());
    //     }
    //     return true;
    //   });
    //   print(
    //       '+++++++++++++++++++ ItemCategory count ${(await DatabaseService.rawQuery('select count(*) from ItemCategory'))}');
    // } else {
    //   print('XXXX Error ItemCategory ${categoryResult.message} ');
    // }

    return true;
  }
}
