part of tecfy_core_client_package;

class DropdownAdvancedEntity extends StatefulWidget {
  final Entity entity;
  final String entityId;
  final String url;
  final String label;
  final bool isRequired;
  final Map<String, dynamic> data;
  final PropertyDropdown property;
  final void Function(String, Map) onChanged;
  DropdownAdvancedEntity(this.entityId,
      {this.property,
      this.label,
      this.isRequired = false,
      this.entity,
      this.url,
      this.onChanged,
      this.data})
      : assert(entity != null);
  @override
  _DropdownAdvancedEntityState createState() => _DropdownAdvancedEntityState();
}

class _DropdownAdvancedEntityState extends AppState<DropdownAdvancedEntity> {
  List<dynamic> listData;

  String get _value => dataValue['id'];
  String _initValue;
  Map<String, dynamic> dataValue;
  Entity _entity;
  String entityPropName;
  dynamic defaultParentData = {'id': null};

  @override
  void initState() {
    super.initState();
    init();
  }

  init() {
    _initValue = widget.data[widget.property.dbName];

    if (_entity == null) _entity = widget.entity;
    if (entityPropName == null)
      entityPropName = widget.property.dbName
          .substring(0, widget.property.dbName.indexOf("Id"));

    if (dataValue == null || dataValue['id'] != _initValue)
      dataValue = widget.data[entityPropName];

    if (dataValue == null && _initValue != null) {
      // load data from server
      EntityService().getOne(widget.entity.dbName, _initValue).then((result) {
        if (result != null) {
          setState(() {
            dataValue = result;
            widget.data[entityPropName] = result;
          });
        }
      });
    }
  }

  _onItemSelectedFromList(context, v) {
    dataValue = v;
    print(dataValue);
    widget.data[entityPropName] = dataValue;
    if (widget.onChanged != null) widget.onChanged(_value, dataValue);
    setState(() {});
    Navigator.pop(context);
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    var validateResult = EntityService.validateEntity(_entity, newDataRaw);
    if (!validateResult.success) {
      AppNavigator.showMessage(context, validateResult.message);
      return;
    }
    var addResult =
        await EntityService.saveEntity(_entity, newDataRaw, context, true);
    if (addResult.success) {
      dataValue = addResult.data;
      widget.data[entityPropName] = dataValue;
      _initValue = dataValue['id'];
      widget?.onChanged(_initValue, dataValue);
      Navigator.pop(context, true); // close the add new item's form
    } else {
      AppNavigator.showMessage(context, addResult.message);
    }
  }

  openSearchDialog() async {
    await AppNavigator.showAppDialog(
        context,
        (listDialogContext) => Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Expanded(child: Container()),
                Container(
                    width: min(500, MediaQuery.of(context).size.width),
                    height: min(
                        800,
                        MediaQuery.of(context).size.height -
                            MediaQuery.of(context).padding.top -
                            MediaQuery.of(context).padding.bottom),
                    child: Card(
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.stretch,
                            children: <Widget>[
                          Container(
                              color: ThemeService.backgroundSecondColor,
                              child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Padding(
                                        padding: EdgeInsets.only(left: 10),
                                        child: Text(
                                            '${widget.label}${widget.isRequired ? '*' : ''}',
                                            style: ThemeService.titleStyle)),
                                    IconButton(
                                        icon: Icon(Icons.close),
                                        onPressed: () =>
                                            Navigator.pop(listDialogContext))
                                  ])),
                          Expanded(
                              child: EntityListSearchComponent(
                            property: widget.property,
                            entity: widget.entity,
                            url: widget.url,
                            data: widget.data,
                            // isFilter: widget.isFilter ,
                            // filterValue: widget.filterValue,
                            // filterField: widget.filterField,
                            urlFilterData: widget.data,
                            onClick: (v) =>
                                _onItemSelectedFromList(listDialogContext, v),
                          )),
                          FlatButton(
                            child: Text(trans('Add New Item'),
                                style: ThemeService.flatButtonTextStyle
                                    .copyWith(
                                        decoration: TextDecoration.underline)),
                            onPressed: () async {
                              Map<String, dynamic> newDataRaw =
                                  new Map<String, dynamic>();
                              var addNewResult =
                                  await AppNavigator.showAppDialog(
                                      listDialogContext,
                                      (addDialogContext) => _getAddNewDialog(
                                          addDialogContext, newDataRaw));
                              if (addNewResult ?? false)
                                Navigator.pop(
                                    listDialogContext); // close the search form
                            },
                          )
                        ]))),
                Expanded(child: Container()),
              ],
            ));
    //loadData();
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Card(
        child: ListView(
      children: <Widget>[
        InLineForm(widget.entityId,
            entity: _entity,
            inPopup: true,
            data: newDataRaw,
            onChanged: (v) {}),
        Padding(
            padding: EdgeInsets.symmetric(horizontal: 20),
            child: Row(
              children: <Widget>[
                Expanded(child: Container()),
                Expanded(
                    child: AppButtonPost(
                        label: trans('Save'),
                        onPressed: () =>
                            _onSaveNewItem(addDialogContext, newDataRaw))),
                SizedBox(width: 10),
                Expanded(
                    child: Container(
                        height: 50.0,
                        child: FlatButton(
                            child: Text(trans('Cancel'),
                                style: ThemeService.flatButtonTextStyle),
                            onPressed: () =>
                                Navigator.pop(addDialogContext, false)))),
                Expanded(child: Container()),
              ],
            )),
      ],
    ));
  }

  @override
  Widget build(BuildContext context) {
    if (_initValue != widget.data[widget.property.dbName] ||
        dataValue == null ||
        dataValue['id'] != _initValue) init();

    return MaterialButton(
        padding: EdgeInsets.all(0),
        onPressed: openSearchDialog,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            SizedBox(height: 10),
            Text('${widget.label ?? ''}${widget.isRequired ? '*' : ''}',
                style:
                    TextStyle(color: ThemeService.textSubColor, fontSize: 11)),
            Text(
                dataValue == null
                    ? trans('Click to Select')
                    : (dataValue[_entity.displayField.dbName] ?? 'N/A'),
                style: TextStyle(color: Colors.black, fontSize: 15)),
            SizedBox(height: 10),
            Container(height: 1, color: ThemeService.textSubColor),
            SizedBox(height: 5),
          ],
        ));
  }
}
