part of tecfy_core_client_package;

class SettingPage extends StatefulWidget {
  @override
  _SettingPageState createState() => _SettingPageState();
}

class _SettingPageState extends AppState<SettingPage> {
  int unPostedCount = 0;
  bool syncing = false;

  initState() {
    TecfySyncService.getUnpostedCount().then((r) {
      unPostedCount = r;
      setState(() {});
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context, AppLocale.translate(context, 'Settings'),

      //backgroundColor: ThemeService.backgroundColor,
      // bottomNavigationBar: MainBottomNavigationBar(4,true),
      child: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            SizedBox(height: 15.0),
            _getItem(
                'Language',
                DropdownButtonHideUnderline(
                    child: DropdownButton<Language>(
                  value: LocalConfigService.language,
                  items: [
                    //DropdownMenuItem(child: Text('N/A'),value: 0),
                    DropdownMenuItem(
                        child: Text('عربي'), value: Language.arabic),
                    DropdownMenuItem(
                        child: Text('English'), value: Language.english)
                  ],
                  onChanged: (v) async {
                    LocalConfigService.language = v;
                    //  await ConfigService.saveConfig();
                    //TecfyMainApplication.restartApp(context);
                    // var result = await AppNavigator.confirmMessage(context,
                    //     'Do you want to restart the application to apply the changes?');
                    // if (result) {
                    //   SystemChannels.platform.invokeMethod('SystemNavigator.pop');
                    // }
                    setState(() {});
                  },
                ))),
            SizedBox(height: 15.0),
            _getItem(
                'Connection Status',
                Row(
                  children: <Widget>[
                    Text(trans('Status')),
                    Text(':'),
                    SizedBox(width: 15.0),
                    ConnectionService.getConnectionIconStream(),
                  ],
                )),
            SizedBox(height: 15.0),
            kIsWeb ? Container() : _getItem(
                'Unposted Items',
                Row(
                  children: <Widget>[
                    Text(trans('Count')),
                    Text(':'),
                    SizedBox(width: 15.0),
                    Text(unPostedCount.toString()),
                    SizedBox(width: 30.0),
                    syncing ? CircularProgressIndicator() : Container(),
                    syncing || ConnectionService.isOffline
                        ? Container()
                        : IconButton(
                            tooltip: trans('Sync Now'),
                            icon: Icon(Icons.sync_alt),
                            onPressed: () async {
                              setState(() => syncing = true);
                              await TecfySyncService.syncNow();
                              unPostedCount =
                                  await TecfySyncService.getUnpostedCount();
                              setState(() => syncing = false);
                            })
                  ],
                )),
            SizedBox(height: 15.0),
            _getItem(
                'Account Settings',
                Row(
                  children: <Widget>[
                    Text(trans('Name')),
                    Text(':'),
                    SizedBox(width: 15.0),
                    Text(LocalConfigService.user?.fullName ?? trans("Guest")),
                    SizedBox(width: 30.0),
                    IconButton(
                        tooltip: trans('Update Info'),
                        icon: Icon(Icons.edit),
                        onPressed: () async {
                          await AppNavigator.navigateTo(
                              context, () => AccountPage());
                        })
                  ],
                )),
          ],
        ),
      ),
    );
  }

  _getItem(String title, Widget child) {
    return Card(
      margin: EdgeInsets.symmetric(horizontal: 15.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          Container(
            height: 45.0,
            color: ThemeService.backgroundSecondColor,
            child: Padding(
                padding: EdgeInsets.symmetric(horizontal: 10),
                child: Align(
                    alignment: LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                    child: Text(
                      AppLocale.translate(context, title),
                      style: Theme.of(context).textTheme.title,
                    ))),
          ),
          //item('Account Information', ()=>AccountPage())
          Container(
              height: 70.0,
              color: Colors.white,
              margin: EdgeInsets.symmetric(horizontal: 10),
              child: Align(
                  alignment: LocalConfigService.isArabic
                      ? Alignment.centerRight
                      : Alignment.centerLeft,
                  child: child))
        ],
      ),
    );
  }

  getBoarder(widget) {
    return Container(
        height: 60.0,
        decoration: BoxDecoration(
            border: Border.all(color: ThemeService.mainColor),
            borderRadius: BorderRadius.all(Radius.circular(2.0))),
        child: widget);
  }

  divider() {
    return Padding(
        padding: EdgeInsets.symmetric(horizontal: 10.0),
        child:
            Container(height: 1.0, color: ThemeService.backgroundSecondColor));
  }

  item(title, creator) {
    return GestureDetector(
        onTap: creator == null
            ? null
            : () => AppNavigator.navigateTo(context, creator),
        child: Container(
          height: 70.0,
          color: Colors.white,
          child: Align(
              alignment: Alignment.centerLeft,
              child: Row(
                children: <Widget>[
                  SizedBox(width: 10.0),
                  Expanded(
                      child: Text(
                    title,
                    style: Theme.of(context).textTheme.subtitle,
                  )),
                  SizedBox(width: 10.0),
                  Icon(
                    Icons.arrow_right,
                    color: Theme.of(context).textTheme.subtitle.color,
                  ),
                  SizedBox(width: 10.0),
                ],
              )),
        ));
  }

  Future<String> getAppVersion() async {
    // PackageInfo packageInfo = await PackageInfo.fromPlatform();
    return ''; //packageInfo.version;
  }
}
